function [H,Di1,beta,c,Q,Nat,NorQ]=HBW_OOMPMl1_Trg_FFT(f,D,tol,No,pp,eex,icase);
%OHBW_OOMPMl1_Trg_FFT  Hierarchical Block Wise Optimized Orthogonal Matching Pursuit 
%for triginometric dictionaries. The routine implements the 
%method OHBW-OOMP in Ref 1, using FFT to calculate inner 
%porducts with the triginometric atoms.
% 
%OHBW_OOMPMl1_FFT produces an atomic decomposition of a 1 D signal divided into segments.
%At each iteration the algorithms selects
%i)the elements for approximating the segments composing the signal, using OOMP criterion.
%ii)the hierarchical sequence in which the approximation of the blocks is realized.
% You can choose a tolerance for the approximation or/and the total number of atoms to take in. 
% 
%
% Inputs:
%   f       cell array f{n} is a matric with the signal for block n 
%   D       dictionary of normalized atoms 
%   tol     tolerance for the  square error norm  tol=0 by default 
%   No      maximal number of atoms, if the number of chosen atoms
%           equals to No, routine will stop.
%   eex     auxiliar vector to calculate inner producs via fft
%   icase   option for the dictionary: 
%           icase=4 for Sin-Cos, icase=2 for Cos icase=3 for Sin dictionaries
%
% Outputs:
%   H       cell array with the approximation of f
%   Di      cell with the indices of selected atoms for the block
%   beta    cell array with the biorthogonal atoms to the selected ones
%   c       cell array with coefficients of the atomic decomposition
%   Q       cell array with an ortho basis for the selected subspace
%   Nat     Total number of atoms (in case it is not No)
%  
%   
%----------------------------------------------------------
%  Laura Rebollo-Neira
%  http://www1.aston.ac.uk/eas/staff/dr-laura-rebollo-neira/
%----------------------------------------------------------
%
%  Reference:
%
%  1-``Cooperative Pursuit Strategies for sparse
%  approximation by partitioning'',  
%  Signal Processing, Vol 125, 365--375  (2016).
%  
%  2-L. Rebollo-Neira and D. Lowe, "Optimized Orthogonal Matching Pursuit Approach", IEEE
%  Signal Processing Letters, Vol(9,4), 137-140, (2002). 
%
%
name='OHBW-OOMPMl1-Trg-FFT'; %name of routine
%get inputs and setup parameters
[L,N]=size(D);
%delta=1/(Lx*Ly);
delta=1;
NBx=numel(f)
mul=size(f{1},2);
%!!!end
zmax=1;%number of reorthogonalizations
beta=cell(NBx,1);
Q=cell(NBx,1);
c=cell(NBx,1);
%cc=cell(N,1);
qq=cell(NBx,1);
Di1=cell(NBx,1);
Re=f;
%
kq=zeros(NBx,1);
Cp=zeros(NBx,1);
%
%threshold for coefficients
%
tol2=1e-11;   
Nat=0;
for nb=1:NBx
    NorQ{nb}=zeros(N,1);
    if icase==4 NorQ{nb}=zeros(N/2,2);end
    B=Re{nb};
    for ii=1:mul; snor(ii)=norm(Re{nb}(:,ii))^2;end %residual norm of block qp
    nore(nb)=pp*snor.'*sqrt(delta);
    [max_c,q]=AtSelOOMPMl1TrgFFT(B,N,NorQ{nb},pp,eex,icase);
    qq{nb}=q;  
    Cp(nb)=max_c;
    Cpsi(nb)=Cp(nb);
    Q{nb}=D(:,qq{nb});
    norb=norm(Q{nb});
    Q{nb}=Q{nb}/norb;
    beta{nb}=Q{nb}/norb;
end
%
%===============================
% Main algorithm 
%===============================
while Nat < No 
  %finding of maximal coefficient
    [Cpmax,qb]=max(Cpsi);
    q(1)=qq{qb};  %change from 2D
  %stopping criterion (coefficient)
    if Cpmax<tol2 
      fprintf('%s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',name,tol2);
      break;
    end
    kq(qb)=kq(qb)+1;
    k=kq(qb);
    Di1{qb}=[Di1{qb} q(1)]; %store the indices of selected atoms
  if k>1
  new_atom=D(:,q(1)); 
  nork=norm(Q{qb}(:,k));
  Q{qb}(:,k)=Q{qb}(:,k)/nork;
  beta{qb}=beta{qb}-Q{qb}(:,k)*(new_atom'*beta{qb})/nork;
  beta{qb}(:,k)=Q{qb}(:,k)/nork;
  end
%
  h=Q{qb}(:,k)*(Q{qb}(:,k)'*f{qb});
  Re{qb}=Re{qb}-h;%residual of block qp
  for ii=1:mul; snor(ii)=norm(Re{qb}(:,ii))^2;end %residual norm of block qp
  nore(qb)=pp*snor.'*sqrt(delta);
  c{qb}=f{qb}'*beta{qb};
  NorQ{qb}=Auxi(NorQ{qb},Q{qb}(:,k)',N,eex,icase);
   [gt1]=find(NorQ{qb}(:,1)>=1);
   NorQ{qb}(gt1,1)=0; %sloppy to delate the already chosen atoms
  if icase==4
    [gt2]=find(NorQ{qb}(:,2)>=1);
     NorQ{qb}(gt2,2)=0; %sloppy to delate the already chosen atoms
  end
%
%  calculates inner products via fft 
  [max_c,q]=AtSelOOMPMl1TrgFFT(Re{qb},N,NorQ{qb},pp,eex,icase);
  qq{qb}=q;  
  Cp(qb)=max_c;
  q(1)=qq{qb}(1);
%=================optimiztion of HBW-OMP==============================
  new_atom=D(:,q(1));
   for p=1:k
     QQ=new_atom-(Q{qb}(:,p)'*new_atom)*Q{qb}(:,p); %orthogonalization
  end
   for zi=1:zmax
    for p=1:k
     QQ=QQ-(Q{qb}(:,p)'*QQ)*Q{qb}(:,p); %re-orthogonalization
    end
   end
  Q{qb}(:,k+1)=QQ;
  QQ=QQ/norm(QQ);
  Cpsi(qb)=norm(QQ'*Re{qb},'fro')^2;
% 
  Nat=Nat+1;
  max_err=sum(nore);
  %stopping criterion (distance)
  if (tol~= 0) & (max_err < tol) break;end;
end
%!!!approximation and selected atoms
H=cell(NBx,1);
for i=1:NBx
BH=zeros(L,1);  
H{i}=zeros(L,1); 
id=numel(Di1{i});
for n=1:id;
BH=BH+D(:,Di1{i}(n))*c{i}(:,n)';
end
H{i}=BH;  
end
end
%
%Copyright (C) 2015  Laura REBOLLO-NEIRA, Mathematics Department, Aston University
%
%This program is free software; you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation; either 
%version 2 of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
%without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with this program;
%if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%Boston, MA  02110-1301, USA.
